
class Hermes::Api::Dmo::HousingRequestSharesController < Hermes::Api::Dmo::ApplicationController

  class << self
    def mutate_params
      {
        housing_request_id: {type: 'String'},
        lessor_id: {type: 'String'},
      }
    end 

    def acceptable_params
      {
        code: {type: 'String'},
        expire_at: {type: 'DateTime'},
      }.merge(self.mutate_params).merge(super)
    end 

    def valid_operators
      %w[$in $nin $ne $gt $gte $lt $lte $eq $regex $all]
    end

    def regex_params
      {}
    end 
  end

  def find_by_housing_request_id
    return render_json_error(:unprocessable_entity, :params_housing_request_id_not_found) unless params[:housing_request_id].present?
    
    @housing_request_share = Hermes::HousingRequestShare.order_by(expire_at: :desc).find_by(housing_request_id: params[:housing_request_id])
    return render_json_error(:not_found, :instance_not_found) unless @housing_request_share
    return render_json_error(:unauthorized, :no_access) unless can_access?

    render jsonapi: @housing_request_share
  end

  def can_access?
    if current_user.auth_roles&.include?('superadministrateur') || current_user.roles&.include?('Administrator')
      return true
    end

    if current_user.auth_roles&.include?('bailleur') && @housing_request_share.lessor_id.to_s == current_user.lessor_id.to_s
      return true
    end

    false
  end
end 
