# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des méthodes
class Hermes::Api::Dmo::UsersController < Hermes::Api::Dmo::ApplicationController
  require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s

  class << self
    def mutate_params
      {

      }
    end 

    def acceptable_params
      {
        first_name: {type: 'String'},
        last_name: {type: 'String'},
        email: {type: 'String'},
        gender: {type: 'String'},
        phone_number: {type: 'String'},
        auth_roles: {type: 'Array'},
        inactive_at: {type: 'DateTime'},
        current_sign_in_at: {type: 'DateTime'},
        last_sign_in_at: {type: 'DateTime'},
      }.merge(self.mutate_params).merge(super)
    end 

    def valid_operators
      %w[$in $nin $ne $gt $gte $lt $lte $regex]
    end

    def regex_params
      {

      }
    end 
  end

  NO_ACL = [
    "superadministrateur",
    "collaborateur offre logement",
    "cre",
    "rdsl",
    "entreprise",
    "bailleur",
  ]

  def current_user_use_acl 
    (NO_ACL & (current_user&.auth_roles || [])).empty?
  end 

  def can_see_all_users
    !(current_user_roles & ['Administrator', 'superadministrateur']).empty?
  end 

  def initialize
    super
    @model = User
    @serializer = Hermes::UserSerializer
  end

  def show 
    @instance = 
     if params[:id] == 'me'
        current_user
      elsif params[:id]
        @model.find_by(id: params[:id], **@query_precision)
      end 
      
    return render_json_error(:not_found, :object_not_found) if !@instance
    return render_json_error(:unauthorized, :cannot_see_users) if @instance.id != @user.id && !can_see_all_users

    render(
      jsonapi: @instance,
      serializer: @serializer,
      **@options
    )
  end 

  def index
    return render_json_error(:unauthorized, :cannot_see_users) unless can_see_all_users

    acceptable_params *self.class.acceptable_params.keys
    valid_operators *self.class.valid_operators

    paginate(
      index_aggregation(no_acl: !current_user_use_acl),
      each_serializer: @serializer,
      **@options
    )
  end

  def switch
    @target = User.find_by(id: params[:id])
    @target = User.find_by(main_applicant_id: params[:id]) || Hermes::Applicant.find_by(id: params[:id])&.user unless @target
    return render(json: { success: false, message: 'Utilisateur target non trouvé.' }, status: :not_found) unless @target
    return render nothing: true, status: :not_found unless @user

    if can_switch?
      if targeatable?
        success_switch_response
      else
        render json: {
          success: false,
          message: 'L\'utilisateur target doit être demandeur, bailleur ou entreprise.'
        }, status: :unprocessable_entity
      end
    else
      render json: {
        success: false,
        message: 'Vous n\'avez pas les droits nécessaire pour effectuer cette action.'
      }, status: :unauthorized
    end
  rescue ActionController::ParameterMissing => e
    render_json_error(:not_acceptable, :parameter_missing, { message: e.message })
  end

  def invalidate_token
    return render_json_error(:unprocessable_entity, :missing_params) if params[:token].blank?
    decoded_token = User.decode_jwt_token(bearer_token)
    user_token = Hermes::UserInvalidatedToken.new(user_id: current_user.id, token: params[:token], expiring_at: decoded_token.first['exp']) 
    return render(json: user_token.errors, status: :unprocessable_entity) unless user_token.save

    render(json: { message: 'Token invalidated'}, status: :created)
  end 

  private

  def can_switch?
    current_user_roles.include?('superadministrateur') ||
      current_user_roles.include?('collaborateur offre logement') ||
      current_user_roles.include?('cre')
  end

  def targeatable?
    return true if current_user_roles.include?('superadministrateur') || current_user_roles.include?('cre')

    (%w[demandeur bailleur entreprise] & @target.attributes.slice(:roles, :auth_roles).values.flatten.uniq.compact).any?
  end

  def success_switch_response
    token = @target.generate_jwt(true_user_id: @user.id.to_s)
    render json: {
      success: true,
      info: t(:registered),
      email: @target.email,
      login: @target.login,
      jwt_token: token
    }, status: :ok
  end
end
