# frozen_string_literal: true

module Hermes
  # HousingApplication serializer
  class HousingApplicationSerializer < Hermes::ApplicationSerializer
    type 'housing_applications'

    attributes %i[
      als_id
      applicant_id
      applicant_used_for_siret_birth_date
      applicant_used_for_siret_email
      applicant_used_for_siret_firstname
      applicant_used_for_siret_lastname
      applicated_position
      business_cotation
      cotations
      date_event
      effort_rate
      eligible_position
      housing_offer_id
      housing_request
      housing_request_id
      is_active
      main_applicant_first_name
      main_applicant_last_name
      motive_code
      motive_label
      profile_score
      rank
      request_als_id
      requester_birth_date
      requester_email
      requester_firstname
      requester_lastname
      status
      unique_numbers
      visit_conducted_date
      visit_status
    ]

    attribute :applicated_at, unless: :is_light_serializer?
    attribute :housing_offer, unless: :is_light_serializer?
    attribute :housing_request, unless: :is_light_serializer?
    attribute :last_send_has_error, unless: :is_light_serializer?
    attribute :last_status_update_date, unless: :is_light_serializer?
    attribute :last_xml_validation_report, unless: :is_light_serializer?
    attribute :lessor_event_informations, unless: :is_light_serializer?
    attribute :max_date, unless: :is_light_serializer?
    attribute :offer_external_ref, unless: :is_light_serializer?
    attribute :status_updated_at, unless: :is_light_serializer?
    attribute :visit_status_updated_at, unless: :is_light_serializer?
    attribute :applicant, if: :embedded_applicant?

    def is_light_serializer?
      @instance_options[:lignt_serializer]
    end 
    
    REQUEST_ATTRIBUTES = %w[
      seeked_localisation_list
      actual_housing_details
    ]
    
    REQUEST_ATTRIBUTES.each do |att|
      attribute att.to_sym
      define_method(:"#{att}") do
        object&.housing_request[att]
      end
    end 

    def added_to_object_cache_key
      object&.housing_request&.updated_at.to_i.to_s + @instance_options.to_s
    end 

    def motive_label
      HERMES_CONFIG['motive_label'] && HERMES_CONFIG['motive_label'][object.motive_code]
    end 
    
    def offer_external_ref
      object.housing_offer&.external_ref
    end
    
    def main_applicant_first_name
      object.housing_request.main_applicant.first_name
    end
    
    def main_applicant_last_name
      object.housing_request.main_applicant.last_name
    end
    
    def request_als_id
      object.housing_request.als_id
    end
    
    def unique_numbers
      object.housing_request&.nuds&.pluck(:unique_number)
    end
    
    def embedded_applicant?
      @instance_options[:embedded_applicant] == true
    end
    
    def applicant_id
      object.applicant_id&.to_s
    end
    
    def applicant
      return unless object.applicant
      
      Hermes::ApplicantSerializer.new(
        object.applicant,
        scope: scope,
        root: false
      )
    end
    
    def housing_offer_id
      object.housing_offer_id&.to_s
    end
    
    def housing_request
      return unless @instance_options[:join_housing_request]
      
      Hermes::HousingRequestSerializer.new(
        object.housing_request,
        scope: scope,
        root: false
      )
    end

    def housing_offer
      return unless @instance_options[:join_housing_offer]
      
      Hermes::HousingOfferSerializer.new(
        object.housing_offer,
        scope: scope,
        root: false
      )
    end
  end
end
