# frozen_string_literal: true

require 'base64'
module Hermes
  module BoClient
    # extending Mapping Module from base_map.rb
    module Mappings
      # handle HousingRequest model mapping for xml data send
      class HousingRequestMap
        include Hermes::BoClient::Mappings::Concerns::BaseMap
        
        def initialize(options)
          housing_request_id = options[:housing_request_id]


          @numUnique = options[:num_unique]
          @pieceIDDl = options[:pieceID]
          @required_document = options[:required_document_id] ? Hermes::RequiredDocument.find_by(id: options[:required_document_id]) : false
          @main_object = Hermes::HousingRequest.find_by(id: housing_request_id)
          @nud = options[:nud_id] ? Hermes::Nud.find_by(id: options[:nud_id]) : false
          @codes_orientation_intiale = options[:codes_orientation_intiale]
          self.main_object = @main_object
          @map = {
            
            
            
            "ressourcesLoiEC": 'ec_law_ressources',
            "montantLoyer": 'seeked_max_rent',
            
            "compNomOrgBailleur": 'lessor_name',
            "compSIRENOrgBailleur": 'lessor_siren',
            "codePostalLoc": 'seeked_district_postal_codes',
            "nom": 'requester_lastname',
            "prenom": 'requester_firstname',
            
            "mel": 'requester_email',
            
            "anneeMoins1": 'main_applicant.year_minus_one',
            "anneeMoins2": 'main_applicant.year_minus_two',
            
            "listeRessourceRecue": 'main_applicant&.monthly_incomes',
            "listeRessourceVersee": 'main_applicant&.monthly_charges',
            
            "listeRessourceRecuePersACharge": 'dependents',
            "listeRessourceVerseePersACharge": 'dependents'
            
          }
        end
        
        {
          
          
          "montantMensuelAPL": "main_applicant&.actual_housing_details&['APL_monthly_amount']",
          "nombreHabitant": "main_applicant&.actual_housing_details&['number_of_resident']",
          "surface": "main_applicant&.actual_housing_details&['housing_surface']"
        }.each do |key, val|
          define_method key.to_sym do
            s = val.split("&['")
            splitted = []
            s.each do |str|
              splitted << str.split("']")[0]
            end
            if splitted.length == 2
              if splitted[0].include? 'main_applicant&.actual_housing_details'
                return @main_object.main_applicant&.actual_housing_details ? @main_object.main_applicant&.actual_housing_details[splitted[1]] : nil
              elsif @main_object.send(splitted[0])
                return @main_object.send(splitted[0])[splitted[1]]
              end
            end
            if splitted.length == 3
              if splitted[0].include? 'main_applicant&.actual_housing_details'
                return @main_object.main_applicant&.actual_housing_details ? @main_object.main_applicant&.actual_housing_details[splitted[1]][splitted[2]] : nil
              elsif @main_object.send(splitted[0]) && @main_object.send(splitted[0])[splitted[1]]
                return @main_object.send(splitted[0])[splitted[1]][splitted[2]]
              end
            end
          end
        end
        def refInterne
          @main_object.reload.als_id || ""
        end
        def xml
          return @main_object.get_local_xml
        end
        def listeHandicap
          @main_object.disability_details
        end
        
        def personneANaitre
          @main_object.unborn_children
        end
        
        def listeModeleLogement
          @main_object.housing_type_list || []
        end
        
        def nir
          @main_object.main_applicant&.ss_id
        end
        
        def melAssistantDemarches
          @main_object.main_applicant&.email_advances_assistant
        end
        
        def nomPatronymique
          @main_object.main_applicant&.last_name
        end

        def prenoms
          @main_object.main_applicant&.first_name
        end
        
        def nomEmployeurPrincipal
          @main_object.main_applicant&.main_employer_name
        end
        
        def siretEmployeur
          @main_object.main_applicant&.main_employer_siret
        end
        
        def listeRessourceRecuePersACharge
          @main_object.dependents
        end
        
        def listeRessourceVerseePersACharge
       
          @main_object.dependents
        end
        
        def listePersonneACharge
          @main_object.dependents
        end
        
        def listeRessourceRecue
          @main_object.main_applicant&.monthly_incomes
        end
        
        def listeRessourceVersee
          @main_object.main_applicant&.monthly_charges
        end
        
        def profession
          @main_object.main_applicant&.profession_details&.fetch('profession', nil)
        end
        
        def etrangerProf
          if @main_object.main_applicant&.profession_details&.fetch('foreign', nil) && @main_object.main_applicant&.profession_details&.fetch('foreign', nil) != 'false'
            1
          else
            0
          end
        end
        
        def codePostalProf
          @main_object.main_applicant&.workplace_postal_code
        end
        
        def communeProf
          @main_object.main_applicant&.workplace_insee_code
        end
        
        def telTravail
          @main_object.main_applicant&.workphone
        end
        
        def telPortable
          @main_object.main_applicant&.cellphone
        end
        
        def telDomicile
          @main_object.main_applicant&.homephone
        end
        
        def localisationSouhaite
          list = @main_object.seeked_localisation_list
          res = []
          list&.each do |item|
            item['codesPostaux']&.each do |codepostal|
              res_item = {
                code: item['code'],
                codePostal: codepostal,
                seeked_epci_first_quartil: item['seeked_epci_first_quartil'],
                seeked_quartil_belongings: item['seeked_quartil_belongings']
                
              }
              res.push(res_item)
            end
          end
          res
        end
        
        def avecParking
          @main_object.with_car_park ? 1 : 0
        end
        
        def appartementLog
          @main_object&.actual_housing_details&.fetch('number', nil)
        end
        
        def etageLog
          @main_object&.actual_housing_details&.fetch('floor', nil)
        end
        
        def escalierLog
          @main_object&.actual_housing_details&.fetch('staircase', nil)
        end
        
        def batimentLog
          @main_object&.actual_housing_details&.fetch('building', nil)
        end
        
        def compAdresseLog
          @main_object&.actual_housing_details&.fetch('company_adress', nil)
        end
        
        def lieuDitLog
          @main_object&.actual_housing_details&.fetch('address_place', nil)
        end
        
        def numeroLog
          @main_object&.actual_housing_details&.fetch('number', nil)
        end
        
        def nomHebergeantLog
          @main_object&.actual_housing_details&.fetch('houser_name', nil)
        end
        
        def appartementDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('appartment_number', nil)
          else
            @main_object.actual_housing_details&.fetch('appartment', nil)
          end
        end
        
        def etageDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('appartment_floor', nil)
          else
            @main_object.actual_housing_details&.fetch('floor', nil)
          end
        end
        
        def escalierDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('appartment_staircase', nil)
          else
            @main_object.actual_housing_details&.fetch('appartment_staircase', nil)
          end
        end
        
        def batimentDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('appartment_building', nil)
          else
            @main_object.actual_housing_details&.fetch('appartment_building', nil)
          end
        end
        
        def compAdresseDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('address_add_address', nil)
          else
            @main_object.actual_housing_details&.fetch('address_add_address', nil)
          end
        end
        
        def nomHebergeantDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('houser_name', nil)
          else
            @main_object.actual_housing_details&.fetch('houser_name', nil)
          end
        end
        
        def lieuDitDem
          if @main_object.postal_details && @main_object.postal_details != {}
            
            @main_object.postal_details&.fetch('appartment_address_place', nil)
          else
            @main_object.actual_housing_details&.fetch('appartment_address_place', nil)
          end
        end
        
        def numeroDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('address_number', nil)
          else
            @main_object.actual_housing_details&.fetch('number', nil)
          end
        end
        
        def voieDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('address_street', nil)
          else
            @main_object.actual_housing_details&.fetch('address_street', nil) || @main_object.actual_housing_details&.fetch('way', nil)
          end
        end
        
        def numGuichet
          HERMES_CONFIG['code_guichet'] || 'G1175'
        end
        
        def codePostalDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('postal_code', nil)
          else
            @main_object.actual_housing_details&.fetch('postal_code', nil)
          end
        end
        
        def codePostalEtrangerDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('current_address_postal_code', nil)
          else
            @main_object.actual_housing_details&.fetch('current_address_postal_code', nil)
          end
        end
        
        def communeEtrangerDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('foreign_city', nil)
          else
            @main_object.actual_housing_details&.fetch('foreign_city', nil)
          end
        end
        
        def paysDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('foreign_country', nil)
          else
            @main_object.actual_housing_details&.fetch('foreign_country', nil)
          end
        end
        
        def communeDem
          if @main_object.postal_details && @main_object.postal_details != {}
            @main_object.postal_details&.fetch('address_insee_code', nil)
          else
            @main_object.actual_housing_details&.fetch('address_insee_code', nil)
          end
        end
        
        def codePostalLog
          @main_object&.actual_housing_details&.fetch('postal_code', nil)
        end
        
        def codePostalEtrangerLog
          @main_object&.actual_housing_details&.fetch('current_address_postal_code', nil)
        end
        
        def communeEtrangerLog
          @main_object&.actual_housing_details&.fetch('foreign_city', nil)
        end
        
        def paysLog
          @main_object&.actual_housing_details&.fetch('foreign_country', nil)
        end
        
        def codePostalEtrangerProf
          @main_object.main_applicant&.foreign_workplace_postal_code
        end
        
        def communeEtrangerProf
          @main_object.main_applicant&.foreign_workplace_locality
        end
        
        def paysProf
          @main_object.main_applicant&.foreign_workplace_country
        end
        
        def communeLog
          @main_object&.actual_housing_details&.fetch('address_insee_code', nil)
        end
        
        def voieLog
          @main_object&.actual_housing_details&.fetch('way', nil) || @main_object.main_applicant&.actual_housing_details&.fetch('way', nil)
        end
        
        def forcerDoublon
          @main_object.forcing_duplicate ? 1 : 0
        end
        
        def modeleLogement
          @main_object&.actual_housing_details&.fetch('housing_typology', 'T1') || 'T1'
        end
        
        def montantMensuelLoyer
          @main_object&.actual_housing_details&.fetch('monthly_rental_amount', '0') || '0'
        end
        
        def proprietaire
          @main_object.is_house_owner ? 1 : 0
        end
        
        def demandeElargie
          @main_object.widen_request ? 1 : 0
        end
        
        def rezDeChaussee
          @main_object.seeking_first_floor ? 1 : 0
        end
        
        def sansAscenseur
          @main_object.seeking_elevator ? 1 : 0
        end
        
        def multiEmployeur
          @main_object.main_applicant&.has_other_employers ? 1 : 0
        end
        
        def renouvellementElec
          return 1 if @main_object.electronical_renewal.nil?
          
          @main_object.electronical_renewal ? 1 : 0
        end
        
        def precision
          @main_object.precisions || ''
        end
        
        def dateCreationDemande
          if @main_object.respond_to?("sne_created_at") && @main_object.sne_created_at
            @main_object.sne_created_at&.to_date
          else
            DateTime.now.to_date
          end
        end
        
        def dateNaissance
          @main_object.main_applicant.birth_date&.to_date
        end
        
        def montantMoins1
          if @main_object.main_applicant.amount_year_minus_one
            @main_object.main_applicant.amount_year_minus_one&.round.to_s
          end
        end
        
        def montantMoins2
          if @main_object.main_applicant.amount_year_minus_two
            @main_object.main_applicant.amount_year_minus_two&.round.to_s
          end
        end
        
        def etrangerDem
          if @main_object.postal_details && @main_object.postal_details['etranger'] && @main_object.postal_details['etranger'] != 'false'
            1
          else
            0
          end
        end
        
        def etrangerLog
          if @main_object.main_applicant&.actual_housing_details && @main_object.main_applicant&.actual_housing_details['etranger'] && @main_object.main_applicant&.actual_housing_details['etranger'] != 'false'
            1
          else
            0
          end
        end
        
        def etrangerLogAct
          if @main_object.main_applicant&.actual_housing_details && @main_object.main_applicant&.actual_housing_details['etranger'] && @main_object.main_applicant&.actual_housing_details['etranger'] != 'false'
            1
          else
            0
          end
        end
        
        def categorieLogementAct
          @main_object.main_applicant&.actual_housing_details&.fetch('housing_category_list', nil)
        end
        
        def typeLogementActuelAct
          @main_object.main_applicant&.actual_housing_details&.fetch('typeLogementActuel', nil)
        end
        
        def communeLogAct
          @main_object.main_applicant&.actual_housing_details&.fetch('address_insee_code', nil)
        end
        
        def codePostalLogAct
          @main_object.main_applicant&.actual_housing_details&.fetch('postal_code', nil)
        end
        
        def typeLogementPossede
          @main_object.main_applicant&.owned_housing_details&.fetch('type', nil)
        end
        
        def typeLogement
          @main_object.seeked_types
        end
        
        def civilite
          @main_object.main_applicant&.civility
        end
        
        def nationalite
          @main_object&.main_applicant&.nationality
        end
        
        def situationFamiliale
          @main_object&.main_applicant&.family_situation
        end
        
        def categorieLogementRecherche
          res = @main_object&.seeked_housing_categories&.fetch(0, nil) || 'J'
          return 'J' if res == ''
          
          res
        end
        
        def typeContratTravail
          @main_object&.main_applicant&.work_contract_type
          # return "RET"
        end
        
        def motifDemande
          @main_object&.motivations
        end
        
        def notifSms
          unless @main_object.main_applicant&.sms_notification.nil?
            @main_object.main_applicant&.sms_notification ? 1 : 0
          end
        end
        
        def notifEvt
          unless @main_object.main_applicant&.event_notification.nil?
            @main_object.main_applicant&.event_notification ? 1 : 0
          end
        end
        
        def numUnique
          if @numUnique && @numUnique.length == 18
            @numUnique
          elsif @nud && @nud.unique_number.length == 18
            @nud.unique_number
          else
            @main_object.unique_numbers.last
          end
        end
        
        def coapplicants
          @main_object.coapplicants
        end
        
        def lienDemandeur(i)
          @main_object.coapplicants[i][:rel_type]
        end
        
        attr_reader :pieceIDDl
        
        def codeCategoriePiece
          @required_document.sne_category
        end
        
        def idPiece
          @required_document&.sne_piece_id
        end
        
        def typePiece
          @required_document.sne_code
        end
        
        def dateFonctionnellePiece
          Date.current
        end
        
        def sensibilitePiece
          0
        end
        
        def documentInternePiece
          0
        end
        
        def dateCreationPiece
          Date.current
        end
        
        def indicePertinence
 
            "NV"
       
        
        end
        
        def dateDerniereActionPiece
          Date.current
        end
      
        
        def idREX
          @main_object&.entity_id
        end
        
        def appCommerciale
          @main_object&.entity&.title
        end
        
        def numSiret
          @main_object&.entity&.siret
        end
        
        def codes_orientation_intiale
          @codes_orientation_intiale
        end
        
        def date_orientation
          Date.current
        end
      end
    end
  end
end
