require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::NudProvidersController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers

  check_mutate_params(
    model: Hermes::NudProvider, 
    mutate_params: Hermes::Api::Dmo::NudProvidersController.mutate_params, 
    factory: :nud_provider,
    user_roles: ['Administrator']
  )


  let!(:responsible) {create(:user, roles: ['Administrator'])}
  let!(:user) {create(:user)}

  #TESTS BASIQUES D'API

  describe 'POST /nud_providers' do
    describe 'Create nud_provider with minimal valid datas' do
      before :each do
        authenticated_header(request, responsible)
        post 'create', {
            data: {
                attributes: {
                }
            }
        }
      end
      it 'returns status code 201' do
        expect(response).to have_http_status(201)
      end
    end
    
    
  end

  describe 'GET /nud_providers' do
    describe 'GET /nud_providers' do
      let!(:nud_provider1) {create(:nud_provider)}
      let!(:nud_provider3) {create(:nud_provider)}
      let!(:nud_provider2) {create(:nud_provider)}

      before :each do
        authenticated_header(request, responsible)
        get 'index'
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(3)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(3)
      end
    end
  end

  describe 'GET /nud_providers/:id' do
    describe 'verif it work' do
      let!(:nud_provider1) {create(:nud_provider)}
      before :each do
        authenticated_header(request, responsible)
        get 'show', {id: nud_provider1.id}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
      end
    end

  end


  describe 'GET /nud_providers/:departement' do
    describe 'verif it work with 2digit' do
      let!(:nud_provider1) {create(:nud_provider, department_codes: ["33"])}
      before :each do
        authenticated_header(request, responsible)
        get 'show', {id: "33"}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
      end
    end
    describe 'verif it work with 3digit' do
      let!(:nud_provider1) {create(:nud_provider, department_codes: ["976"])}
      before :each do
        authenticated_header(request, responsible)
        get 'show', {id: "976"}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
      end
    end
    describe 'verif not found' do
      let!(:nud_provider1) {create(:nud_provider, department_codes: ["976"])}
      before :each do
        authenticated_header(request, responsible)
        get 'show', {id: "00"}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(404)
      end
    end
  end

  describe 'PATCH /nud_providers/:id' do

  end

  describe 'DELETE /nud_providers/:id' do
    let!(:nud_provider) {create(:nud_provider, user_id: responsible.id.to_s)}
    before :each do
      authenticated_header(request, responsible)

      delete 'destroy', {
          id: nud_provider.id.to_s
      }
    end
    it 'returns status code 204' do
      expect(response).to have_http_status(204)
    end
  end

end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::NudProvidersController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_NUD_PROVIDERS'
  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:admin) { create(:user, roles: ['Administrator']) }

  path '/api/dmo/nud_providers' do
    get 'Retrieves all nud_providers' do
      tags tags
      produces 'application/json'

      Hermes::Api::Dmo::NudProvidersController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '200', 'nud_providers found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let!(:nud_providers) { create_list(:nud_provider, 5, name: 'test') }

        include_context 'with integration test'
        include_context 'valid jsonapi collection' do
          let(:expected_collection) { nud_providers }
        end
      end
    end
  end

  path '/api/dmo/nud_providers' do
    post 'Create nud_provider' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::NudProvidersController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
          {  name: 'test' }
        )
      end

      response '201', 'nud_provider created' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)

          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'nud_providers'
        end
      end
    end
  end

  path '/api/dmo/nud_providers/{id}' do
    get 'Retrieves a single nud_provider by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        let(:id) { 1 }

        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }

        run_test!
      end 
      
      response '200', 'nud_provider found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let(:nud_provider) {create(:nud_provider, name: 'test')}
        let(:id) {nud_provider.id}

        include_context 'with integration test'
        include_context 'valid jsonapi ressource' do
          let(:expected_ressource) {nud_provider}
        end
      end
    end
  end

  path '/api/dmo/nud_providers/{id}' do
    put 'Update nud_provider' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::NudProvidersController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter name: :id, in: :path, type: :string
      let(:id) { create(:nud_provider, name: 'test', user: admin).id }
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
          {  name: 'test' }
        )
      end

      response '200', 'nud_provider updated' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)
          
          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'nud_providers'
        end
      end
    end
  end

  path '/api/dmo/nud_providers/{id}' do
    delete 'Delete a single nud_provider by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      response '204', 'nud_provider deleted' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { create(:nud_provider, name: 'test').id }
        run_test!
      end
    end
  end

end