# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'

dir_path = File.dirname(__FILE__)
schema_path = [dir_path, 'radierDemande.xsd'].join('/')
require [Hermes::Engine.root, %w[lib hermes bo_client tasks]].flatten.join('/').to_s

RSpec.describe Hermes::BoClient::CancelRequestTask, type: :helper do
  let!(:xsd) { Nokogiri::XML::Schema(File.read(schema_path)) }
  let!(:hr) { create(:complete_request, als_id: '0000001') }

  it 'should load a valid XSD' do
    expect(xsd).to be_present
    expect(xsd.class).to be Nokogiri::XML::Schema
  end

  describe 'to_xml' do
    it 'as valid xml' do 
      so = Hermes::BoClient::CancelRequestTask.new(housing_request_id: hr.id)
      xml = so.to_xml
      expect(xsd.validate(Nokogiri::XML(xml)).length).to eq(0)
    end 

    it 'should has all values' do
      so = Hermes::BoClient::CancelRequestTask.new(housing_request_id: hr.id)
      parsed_xml = hr.parse_xml(so.to_xml)

      expect(parsed_xml["radierDemande"]).to eq({
        "xmlns:ns2" => "http://v1.als.ikos.webservice.immobilier.sopra/demande/radierDemande",
        "numUnique" => hr.unique_numbers.last,
        "refInterne" => hr.als_id,
      })
    end
  end
end
