require 'rails_helper'
require 'vcr_config'

dir_path = File.dirname(__FILE__)
schema_path = [dir_path, 'StatutOffre.xsd'].join('/')
require [Hermes::Engine.root, %w[lib hermes bo_client tasks]].flatten.join('/').to_s

RSpec.describe Hermes::BoClient::OfferStatusTask, type: :helper do
  
  let!(:xsd) {Nokogiri::XML::Schema(File.read(schema_path))}
  let!(:user1) {create(:user)}
  
  it 'should load a valid XSD' do
    expect(xsd).to be_present
    expect(xsd.class).to be Nokogiri::XML::Schema
  end
  
  context 'with simple housing_offer' do
    let!(:housing_offer1) do
      create(
          :housing_offer,
          offer_status: 'T2',
          external_ref: 'blabli'
      )
    end
    
    let!(:offer_status_task1) do
      build(
          :offer_status_task,
          options: { housing_offer_id: housing_offer1.id}
      )
    end
    
    describe 'using to_xml' do
      it 'should return a valid xml' do
       
        expect(xsd.validate(Nokogiri::XML(offer_status_task1.to_xml))).to eq([])
        offer_status_task1.execute
        VCR.use_cassette("offer_status_task", :match_requests_on => [:method, :uri, :path]) do
          drain_all_jobs
        end
 
        expect(housing_offer1.reload.last_send_has_error).to eq(true)
      end
    end
  end
end