# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'
require [Hermes::Engine.root, %w[lib hermes rex_client tasks]].flatten.join('/').to_s
require [Hermes::Engine.root, %w[lib hermes rex_client]].flatten.join('/').to_s

HERMES_CONFIG['identity_provider_key'] = 'als_hermes_salarie'


RSpec.describe Hermes::RexClient::SyncStructuresTask, type: :helper do
    # let!(:identity_provider) do
    #   create(
    #     :identity_provider,
    #     key: 'als_hermes_salarie',
    #     identifier_rights_url: 'https://rec.client.almerys.com/authz/realms/ALS/protocol/openid-connect/token',
    #     identifier_client_id: 'ALS_HERMES_BACK',
    #     identifier_client_secret: '77ff390e-6cca-4f09-9840-0e799a47a0ac',
    #     identifier_subject_issuer: 'ALS',
    #     account_service_url: 'https://rec.client.almerys.com/auth/realms/ALS/protocol/openid-connect/token',
    #     account_service_client_id: 'ALS_HERMES_SERVICE_ACCOUNT',
    #     account_service_client_secret: '28733e0d-c363-44a2-9f85-ea7d4784a4ad',
    #     create_user_url: 'https://api-rcc.be-ys.com/als-back/rcc/v1/accounts',
    #     add_roles_user_url: 'https://api-rcc.almerys.com/iam/api/authorization/clients/ALS_HERMES_BACK/users/{login}/roles/{role}?realm=ALS',
    #     create_user_headers: {
    #       'Content-Type': 'application/json',
    #       'X-GeXRT-API-Key': '169c24b0-a763-40f7-877d-7f94967ffea4',
    #     }
    #   )
    # end
    # let!(:entity1) {create(:entity)} 
    # let!(:lessor1) {create(:lessor)} 

  # describe 'Sync' do
  #   it 'update user' do
  #     VCR.use_cassette("rex_sync", :match_requests_on => [:method, :uri, :path]) do
  #       client = Hermes::RexClient::Client.new
  #       Hermes::RexClient::SyncStructuresTask.new.sync("2019-04-04", client, true, true)
  #     end
  #     expect(Hermes::Lessor.all().count).to eq(2)  
  #     expect(Hermes::Entity.all().count).to eq(35)  
  #     expect(User.count).to eq(247)  
  #   end
  # end

  describe 'save_log_and_send_email' do
    let!(:tagset) { create(:tagset) }
    let!(:support) {create(:user, email: 'support@plugandwork.fr')}
    
    subject { client.save_log_and_send_email }
    
    context 'default' do 
      let!(:client) { Hermes::RexClient::Client.new }

      it 'not save doc' do
        subject
        expect(Doc.count).to eq(0)
      end
    end 

    context 'with tagset' do
      let!(:client) { Hermes::RexClient::Client.new(tagset_id: tagset.id) }

      it 'save doc' do
        subject
        expect(Doc.count).to eq(2)
      end

      it 'add log and add error' do 
        client.log_puts "line for classic log"
        client.log_puts "line for error", true

        subject

        log = Doc.find_by(type: 'file')
        expect(log.data).to include("line for classic log")
        expect(log.data).to include("line for error")

        email = Doc.find_by(type: 'email')
        expect(email.title).to include('1 erreurs')
        expect(email.data).to include('Les 1 premières erreurs')
        expect(email.data).to include('line for error')
      end 

      it 'with 100+ error' do 
        120.times do |i|
          client.log_puts "line for error #{i}", true
        end 

        subject

        email = Doc.find_by(type: 'email')
        expect(email.title).to include('120 erreurs')
        expect(email.data).not_to include('line for error 101')
        expect(email.data).to include('Pour voir les erreurs suivantes, il faut aller sur ')
      end 

      it 'no error' do 
        subject

        email = Doc.find_by(type: 'email')
        expect(email.data).to include('Aucune erreur')
      end 
    end 

  end 
end
