require 'rails_helper'

RSpec.describe Hermes::XmlSopraError, type: :model do
  it 'has a valid factory' do
    expect(FactoryBot.build(:xml_sopra_error)).to be_valid
  end

  describe 'extracted_error' do
    let!(:xml_sopra_error) {create(:xml_sopra_error)}

    it 'type 1' do 
      xml_sopra_error.set(error_content: "ExceptionWithResponse ----> <?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><ns16:annulerCandidatureReponse xmlns:ns2=\"http://v1.als.ikos.webservice.immobilier.sopra/patrimoineALS\" xmlns:ns3=\"http://v1.als.ikos.webservice.immobilier.sopra/viderCache\" xmlns:ns4=\"http://v1.als.ikos.webservice.immobilier.sopra/bien/transmisBailleur/reponse\" xmlns:ns5=\"http://v1.als.ikos.webservice.immobilier.sopra/candidatsALS\" xmlns:ns6=\"http://v1.als.ikos.webservice.immobilier.sopra/bien/statutOffre/reponse\" xmlns:ns7=\"http://v1.als.ikos.webservice.immobilier.sopra/bien/exportStatutsOffres\" xmlns:ns8=\"http://v1.als.ikos.webservice.immobilier.sopra/bien/exportBiensBailleur\" xmlns:ns9=\"http://v1.als.ikos.webservice.immobilier.sopra/miseADispositionALS\" xmlns:ns10=\"http://v1.als.ikos.webservice.immobilier.sopra/demande/infosComplementaireDemande\" xmlns:ns11=\"http://v1.als.ikos.webservice.immobilier.sopra/pieceJustificative\" xmlns:ns12=\"http://v1.als.ikos.webservice.immobilier.sopra/bien/statutOffre\" xmlns:ns13=\"http://v1.als.ikos.webservice.immobilier.sopra/demande/propositionEvenement\" xmlns:ns14=\"http://v1.als.ikos.webservice.immobilier.sopra/integration/demande/reponse\" xmlns:ns15=\"http://v1.als.ikos.webservice.immobilier.sopra/integration/demande/recherche\" xmlns:ns16=\"http://v1.als.ikos.webservice.immobilier.sopra/demande/annulerCandidatureReponse\" xmlns:ns17=\"http://v1.als.ikos.webservice.immobilier.sopra/demande/infosComplementaireReponse\" xmlns:ns18=\"http://v1.als.ikos.webservice.immobilier.sopra/demande/entrepriseDemandeReponse\" xmlns:ns19=\"http://v1.als.ikos.webservice.immobilier.sopra/demande/annulerCandidature\" xmlns:ns20=\"http://v1.als.ikos.webservice.immobilier.sopra/demande/entrepriseDemande\" xmlns:ns21=\"http://v1.als.ikos.webservice.immobilier.sopra/demande/candidature\" xmlns:ns22=\"http://v1.als.ikos.webservice.immobilier.sopra/demande/candidature/reponse\" xmlns:ns23=\"http://v1.als.ikos.webservice.immobilier.sopra/bien/exportCandidatures\" xmlns:ns24=\"http://v1.als.ikos.webservice.immobilier.sopra/demande/complementDemandeImhoweb\" xmlns:ns25=\"http://v1.als.ikos.webservice.immobilier.sopra/demande/propositionEvenement/reponse\" xmlns:ns26=\"http://v1.als.ikos.webservice.immobilier.sopra/demande/complementDemandeImhowebReponse\" xmlns:ns27=\"http://v1.als.ikos.webservice.immobilier.sopra/integration/patrimoine/reponse\"><erreurs><erreur><code>ERRFIC</code><message>Le Service sopra.immo.biz.ikos.ALS.impl.back.PropositionALSLotDemandeServiceImpl a échoué avec 1 erreur(s): Impossible d'ajouter une étape : le processus est à l'état annulé.</message></erreur></erreurs></ns16:annulerCandidatureReponse>")
      expect(xml_sopra_error.extracted_error).to eq("Le Service sopra.immo.biz.ikos.ALS.impl.back.PropositionALSLotDemandeServiceImpl a échoué avec 1 erreur(s): Impossible d'ajouter une étape : le processus est à l'état annulé.")
    end 

    it 'type 2' do 
      xml_sopra_error.set(error_content: "ExceptionWithResponse ----> <?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><errors xmlns=\"http://webservice.immobilier.sopra/\"><content>Cet enregistrement existe déjà au sein de la base de données (clé : 'ALS', '000001636', '4')</content><error><code>sopra.immobilier.service.business.DuplicateObjectException</code><message><content>Cet enregistrement existe déjà au sein de la base de données (clé : 'ALS', '000001636', '4')</content></message></error></errors>")
      expect(xml_sopra_error.extracted_error).to eq("Cet enregistrement existe déjà au sein de la base de données (clé : 'ALS', '000001636', '4')")
    end 

    it 'type 3' do 
      xml_sopra_error.set(error_content: "GetXMLError ----> #<RestClient::InternalServerError: 500 Internal Server Error>")
      expect(xml_sopra_error.extracted_error).to eq("GetXMLError ----> #<RestClient::InternalServerError: 500 Internal Server Error>")
    end 
  end
end
